.extraActions {
    float: right;
    margin-top: -2.5em;
}

.extraActions form {
    display: inline-block;
}

.fieldPreview {
    background: #eee;
    padding: 1em 1em 1px 1em;
    margin: 0.5em 0;
    -moz-border-radius: 0.5em;
    position: relative;
    border: solid 2px #ccc;
}

.fieldFromBehavior {
    border: dashed 2px #ccc;
}

.fieldFromBehavior .disabled-field-overlay {
    position: absolute;
    top: 0;
    left: 0;
    height: 100%;
    width: 100%;
    z-index: 2;
    background: #fff;
    opacity: 0.6;
}

.fieldLabel {
    float: left;
    background: #ddd;
    border-right: solid 1px #fff;
    border-bottom: solid 1px #fff;
    -moz-border-radius-bottomright: 0.5em;
    margin: -1em -1em 0;
    padding: 0 0.5em;
    position: relative;
    z-index: 3;
}

.fieldFromBehavior .fieldLabel {
    outline-width-top: 0;
    outline-left: none;
}

.fieldControls {
    position: absolute;
    top: 1em;
    right: 1em;
}

#content #form fieldset {
    border-top: none;
}

#content #form legend {
    display: none;
}
